@@ -125,10 +125,6 @@ Booster <- R6::R6Class(
           
           private$has_gp_model = TRUE
           save_data = RJSONIO::fromJSON(content=modelfile)
-          # Do we have a booster_str as character?
-          if (!is.character(save_data[["booster_str"]])) {
-            stop("gpb.Booster: Can only use a string as booster_str in modelfile")
-          }
           # Create booster from string
           handle <- .Call(
             LGBM_BoosterLoadModelFromString_R
@@ -168,17 +164,52 @@ Booster <- R6::R6Class(
         }
         
       } else if (!is.null(model_str)) {
+        # Create booster from string
         
         # Do we have a model_str as character?
         if (!is.character(model_str)) {
           stop("gpb.Booster: Can only use a string as model_str")
         }
         
-        # Create booster from string
-        handle <- .Call(
-          LGBM_BoosterLoadModelFromString_R
-          , model_str
-        )
+        if (substr(model_str, 20, 20) == "1") {
+          # has gp_model
+          
+          private$has_gp_model = TRUE
+          save_data = RJSONIO::fromJSON(content = model_str)
+          # Create booster from string
+          handle <- .Call(
+            LGBM_BoosterLoadModelFromString_R
+            , save_data[["booster_str"]]
+          )
+          # create gp_model from list
+          private$gp_model <- gpb.GPModel$new(model_list = save_data[["gp_model_str"]])
+          if (!is.null(save_data[["raw_data"]])) {
+            
+            private$train_set <- gpb.Dataset(
+              data = matrix(unlist(save_data[["raw_data"]]$data),
+                            nrow = length(save_data[["raw_data"]]$data),
+                            byrow = TRUE),
+              label = save_data[["raw_data"]]$label)
+            
+          } else {
+            
+            if (private$gp_model$get_likelihood_name() == "gaussian") {
+              private$residual_loaded_from_file <- save_data[["residual"]]
+            } else {
+              private$fixed_effect_train_loaded_from_file <- save_data[["fixed_effect_train"]]
+              private$label_loaded_from_file <- save_data[["label"]]
+            }
+            private$gp_model_prediction_data_loaded_from_file <- TRUE
+            
+          }
+          
+        } else { # has no gp_model
+          
+          handle <- .Call(
+            LGBM_BoosterLoadModelFromString_R
+            , model_str
+          )
+        }
         
       } else {
         
@@ -542,16 +573,60 @@ Booster <- R6::R6Class(
       }
       
       # Save gp_model
+      if (private$has_gp_model) {
+        
+        model_str <- self$save_model_to_string(start_iteration = start_iteration,
+                                               num_iteration = num_iteration, 
+                                               feature_importance_type = feature_importance_type,
+                                               save_raw_data = save_raw_data)
+        write(model_str, file=filename)
+        
+        
+      } else {# has no gp_model
+        
+        # Save booster model
+        .Call(
+          LGBM_BoosterSaveModel_R
+          , private$handle
+          , as.integer(num_iteration)
+          , as.integer(feature_importance_type)
+          , filename
+        )
+      }
+      
+      return(invisible(self))
+    },
+    
+    # Save model to string
+    save_model_to_string = function(start_iteration = NULL, num_iteration = NULL,
+                                    feature_importance_type = 0L, save_raw_data = FALSE, ...) {
+      
+      # Check if number of iteration is non existent
+      if (is.null(num_iteration)) {
+        num_iteration <- self$best_iter
+      }
+      # Check if start iteration is non existent
+      if (is.null(start_iteration)) {
+        start_iteration <- 0L
+      }
+      
+      bst_model_str <- .Call(
+        LGBM_BoosterSaveModelToString_R
+        , private$handle
+        , as.integer(start_iteration)
+        , as.integer(num_iteration)
+        , as.integer(feature_importance_type)
+      )
+      
       if (private$has_gp_model) {
         
         if (is.null(private$train_set$.__enclos_env__$private$raw_data)) {
-          stop("gpb.save: cannot save to file.
-                Set ", sQuote("free_raw_data = FALSE"), " when you construct the gpb.Dataset")
+          stop("Cannot save to file or string when " , sQuote("free_raw_data = TRUE"), ".",
+               " Set ", sQuote("free_raw_data = FALSE"), " when you construct the gpb.Dataset")
         }
         save_data <- list()
         save_data[["has_gp_model"]] <- 1L
-        save_data[["booster_str"]] <- self$save_model_to_string(num_iteration = num_iteration,
-                                                                feature_importance_type = feature_importance_type)
+        save_data[["booster_str"]] <- bst_model_str
         save_data[["gp_model_str"]] <- private$gp_model$model_to_list(include_response_data = FALSE)
         
         if (save_raw_data) {
@@ -586,45 +661,14 @@ Booster <- R6::R6Class(
           
         }# end !save_raw_data
         
-        save_data <- RJSONIO::toJSON(save_data, digits=17)
-        write(save_data, file=filename)
+        model_str <- RJSONIO::toJSON(save_data, digits=17)
         
       } else {# has no gp_model
         
-        # Save booster model
-        .Call(
-          LGBM_BoosterSaveModel_R
-          , private$handle
-          , as.integer(num_iteration)
-          , as.integer(feature_importance_type)
-          , filename
-        )
+        model_str <- bst_model_str
+        
       }
       
-      return(invisible(self))
-    },
-    
-    # Save model to string
-    save_model_to_string = function(start_iteration = NULL, num_iteration = NULL,
-                                    feature_importance_type = 0L) {
-      
-      # Check if number of iteration is non existent
-      if (is.null(num_iteration)) {
-        num_iteration <- self$best_iter
-      }
-      # Check if start iteration is non existent
-      if (is.null(start_iteration)) {
-        start_iteration <- 0L
-      }
-      
-      model_str <- .Call(
-        LGBM_BoosterSaveModelToString_R
-        , private$handle
-        , as.integer(start_iteration)
-        , as.integer(num_iteration)
-        , as.integer(feature_importance_type)
-      )
-      
       return(model_str)
       
     },
@@ -662,19 +706,27 @@ Booster <- R6::R6Class(
                        gp_coords_pred = NULL,
                        gp_rand_coef_data_pred = NULL,
                        cluster_ids_pred = NULL,
-                       vecchia_pred_type = NULL,
-                       num_neighbors_pred = -1,
                        predict_cov_mat = FALSE,
                        predict_var = FALSE,
                        cov_pars = NULL,
                        ignore_gp_model = FALSE,
                        rawscore = NULL,
+                       vecchia_pred_type = NULL,
+                       num_neighbors_pred = NULL,
                        ...) {
       
       if (!is.null(rawscore)) {
         stop("predict: The argument 'raw_score' is discontinued. 
              Use the renamed equivalent argument 'pred_latent' instead")
       }
+      if (!is.null(vecchia_pred_type)) {
+        stop("predict: The argument 'vecchia_pred_type' is discontinued. 
+             Use the function 'set_prediction_data' to specify this")
+      }
+      if (!is.null(num_neighbors_pred)) {
+        stop("predict: The argument 'num_neighbors_pred' is discontinued. 
+             Use the function 'set_prediction_data' to specify this")
+      }
       
       # Check if number of iteration is non existent
       if (is.null(num_iteration)) {
@@ -743,9 +795,7 @@ Booster <- R6::R6Class(
                                                          , predict_var = predict_var
                                                          , cov_pars = cov_pars
                                                          , X_pred = NULL
-                                                         , vecchia_pred_type = vecchia_pred_type
-                                                         , num_neighbors_pred = num_neighbors_pred
-                                                         , predict_response = !pred_latent)
+                                                         , predict_response = !pred_latent )
           fixed_effect = predictor$predict( data = data
                                             , start_iteration = start_iteration
                                             , num_iteration = num_iteration
@@ -776,7 +826,7 @@ Booster <- R6::R6Class(
             response_mean <- random_effect_pred$mu + fixed_effect
             fixed_effect <- NULL
           }
-
+          
         }# end Gaussian data
         else{# non-Gaussian data
           
@@ -817,20 +867,18 @@ Booster <- R6::R6Class(
             # Note: we don't need to provide the response variable y as this is saved
             #   in the gp_model ("in C++") for non-Gaussian data. y is only not NULL when
             #   the model was loaded from a file
-            random_effect_pred = private$gp_model$predict(group_data_pred = group_data_pred,
-                                                          group_rand_coef_data_pred = group_rand_coef_data_pred,
-                                                          gp_coords_pred = gp_coords_pred,
-                                                          gp_rand_coef_data_pred = gp_rand_coef_data_pred,
-                                                          cluster_ids_pred = cluster_ids_pred,
-                                                          predict_cov_mat = predict_cov_mat,
-                                                          predict_var = predict_var,
-                                                          cov_pars = cov_pars,
-                                                          X_pred = NULL,
-                                                          vecchia_pred_type = vecchia_pred_type,
-                                                          num_neighbors_pred = num_neighbors_pred,
-                                                          predict_response = FALSE,
-                                                          fixed_effects = fixed_effect_train,
-                                                          y = y)
+            random_effect_pred = private$gp_model$predict( group_data_pred = group_data_pred
+                                                          , group_rand_coef_data_pred = group_rand_coef_data_pred
+                                                          , gp_coords_pred = gp_coords_pred
+                                                          , gp_rand_coef_data_pred = gp_rand_coef_data_pred
+                                                          , cluster_ids_pred = cluster_ids_pred
+                                                          , predict_cov_mat = predict_cov_mat
+                                                          , predict_var = predict_var
+                                                          , cov_pars = cov_pars
+                                                          , X_pred = NULL
+                                                          , predict_response = FALSE
+                                                          , fixed_effects = fixed_effect_train
+                                                          , y = y )
             
             if (length(fixed_effect) != length(random_effect_pred$mu)){
               warning("Number of data points in fixed effect (tree ensemble) and random effect are not equal")
@@ -846,21 +894,19 @@ Booster <- R6::R6Class(
           }# end pred_latent
           else {# predict response variable for non-Gaussian data
             
-            pred_resp <- private$gp_model$predict(group_data_pred = group_data_pred,
-                                                  group_rand_coef_data_pred = group_rand_coef_data_pred,
-                                                  gp_coords_pred = gp_coords_pred,
-                                                  gp_rand_coef_data_pred = gp_rand_coef_data_pred,
-                                                  cluster_ids_pred = cluster_ids_pred,
-                                                  predict_cov_mat = predict_cov_mat,
-                                                  predict_var = predict_var,
-                                                  cov_pars = cov_pars,
-                                                  X_pred = NULL,
-                                                  vecchia_pred_type = vecchia_pred_type,
-                                                  num_neighbors_pred = num_neighbors_pred,
-                                                  predict_response = TRUE,
-                                                  fixed_effects = fixed_effect_train,
-                                                  fixed_effects_pred = fixed_effect,
-                                                  y=y)
+            pred_resp <- private$gp_model$predict( group_data_pred = group_data_pred
+                                                  , group_rand_coef_data_pred = group_rand_coef_data_pred
+                                                  , gp_coords_pred = gp_coords_pred
+                                                  , gp_rand_coef_data_pred = gp_rand_coef_data_pred
+                                                  , cluster_ids_pred = cluster_ids_pred
+                                                  , predict_cov_mat = predict_cov_mat
+                                                  , predict_var = predict_var
+                                                  , cov_pars = cov_pars
+                                                  , X_pred = NULL
+                                                  , predict_response = TRUE
+                                                  , fixed_effects = fixed_effect_train
+                                                  , fixed_effects_pred = fixed_effect
+                                                  , y = y )
             
             response_mean <-  pred_resp$mu
             response_var <-  pred_resp$var
@@ -1172,6 +1218,7 @@ Booster <- R6::R6Class(
 #' 
 #' # See https://github.com/fabsig/GPBoost/tree/master/R-package for more examples
 #' 
+#' \donttest{
 #' library(gpboost)
 #' data(GPBoost_data, package = "gpboost")
 #' 
@@ -1190,7 +1237,7 @@ Booster <- R6::R6Class(
 #' # Train model
 #' bst <- gpboost(data = X, label = y, gp_model = gp_model, nrounds = 16,
 #'                learning_rate = 0.05, max_depth = 6, min_data_in_leaf = 5,
-#'                objective = "regression_l2", verbose = 0)
+#'                verbose = 0)
 #' # Estimated random effects model
 #' summary(gp_model)
 #' 
@@ -1208,7 +1255,6 @@ Booster <- R6::R6Class(
 #' # For Gaussian data: pred$random_effect_mean + pred$fixed_effect = pred_resp$response_mean
 #' pred$random_effect_mean + pred$fixed_effect - pred_resp$response_mean
 #' 
-#' \donttest{
 #' #--------------------Combine tree-boosting and Gaussian process model----------------
 #' # Create Gaussian process model
 #' gp_model <- GPModel(gp_coords = coords, cov_function = "exponential",
@@ -1216,7 +1262,7 @@ Booster <- R6::R6Class(
 #' # Train model
 #' bst <- gpboost(data = X, label = y, gp_model = gp_model, nrounds = 8,
 #'                learning_rate = 0.1, max_depth = 6, min_data_in_leaf = 5,
-#'                objective = "regression_l2", verbose = 0)
+#'                verbose = 0)
 #' # Estimated random effects model
 #' summary(gp_model)
 #' # Make predictions
@@ -1247,13 +1293,13 @@ predict.gpb.Booster <- function(object,
                                 gp_coords_pred = NULL,
                                 gp_rand_coef_data_pred = NULL,
                                 cluster_ids_pred = NULL,
-                                vecchia_pred_type = NULL,
-                                num_neighbors_pred = -1,
                                 predict_cov_mat = FALSE,
                                 predict_var = FALSE,
                                 cov_pars = NULL,
                                 ignore_gp_model = FALSE,
                                 rawscore = NULL,
+                                vecchia_pred_type = NULL,
+                                num_neighbors_pred = NULL,
                                 ...) {
   
   if (!gpb.is.Booster(x = object)) {
@@ -1276,13 +1322,13 @@ predict.gpb.Booster <- function(object,
       , gp_coords_pred = gp_coords_pred
       , gp_rand_coef_data_pred = gp_rand_coef_data_pred
       , cluster_ids_pred = cluster_ids_pred
-      , vecchia_pred_type = vecchia_pred_type
-      , num_neighbors_pred = num_neighbors_pred
       , predict_cov_mat = predict_cov_mat
       , predict_var = predict_var
       , cov_pars = cov_pars
       , ignore_gp_model = ignore_gp_model
       , rawscore = rawscore
+      , vecchia_pred_type = vecchia_pred_type
+      , num_neighbors_pred = num_neighbors_pred
       , ...
     )
   )
@@ -1307,7 +1353,7 @@ predict.gpb.Booster <- function(object,
 #' gp_model <- GPModel(group_data = group_data[,1], likelihood = "gaussian")
 #' bst <- gpboost(data = X, label = y, gp_model = gp_model, nrounds = 16,
 #'                learning_rate = 0.05, max_depth = 6, min_data_in_leaf = 5,
-#'                objective = "regression_l2", verbose = 0)
+#'                verbose = 0)
 #' pred <- predict(bst, data = X_test, group_data_pred = group_data_test[,1],
 #'                 predict_var= TRUE, pred_latent = TRUE)
 #' # Save model to file
@@ -1379,7 +1425,7 @@ gpb.load <- function(filename = NULL, model_str = NULL) {
 #' gp_model <- GPModel(group_data = group_data[,1], likelihood = "gaussian")
 #' bst <- gpboost(data = X, label = y, gp_model = gp_model, nrounds = 16,
 #'                learning_rate = 0.05, max_depth = 6, min_data_in_leaf = 5,
-#'                objective = "regression_l2", verbose = 0)
+#'                verbose = 0)
 #' pred <- predict(bst, data = X_test, group_data_pred = group_data_test[,1],
 #'                 predict_var= TRUE, pred_latent = TRUE)
 #' # Save model to file
@@ -1476,6 +1522,7 @@ gpb.dump <- function(booster, num_iteration = NULL) {
 #' @return numeric vector of evaluation result
 #'
 #' @examples
+#' \donttest{
 #' # train a regression model
 #' data(agaricus.train, package = "gpboost")
 #' train <- agaricus.train
@@ -1502,6 +1549,7 @@ gpb.dump <- function(booster, num_iteration = NULL) {
 #'
 #' # Get L2 values for "test" dataset
 #' gpb.get.eval.result(model, "test", "l2")
+#' }
 #' @export
 gpb.get.eval.result <- function(booster, data_name, eval_name, iters = NULL, is_err = FALSE) {
   
