@@ -131,4 +131,47 @@ namespace GPBoost {
         return val;
     }
 
+    double digamma(double x) {
+        static double c = 8.5;
+        static double euler_mascheroni = 0.57721566490153286060;
+        double r;
+        double value;
+        double x2;
+        CHECK(x > 0);
+        //
+        //  Use approximation for small argument.
+        //
+        if (x <= 0.000001)
+        {
+            value = -euler_mascheroni - 1.0 / x + 1.6449340668482264365 * x;
+            return value;
+        }
+        //
+        //  Reduce to DIGAMA(X + N).
+        //
+        value = 0.0;
+        x2 = x;
+        while (x2 < c)
+        {
+            value = value - 1.0 / x2;
+            x2 = x2 + 1.0;
+        }
+        //
+        //  Use Stirling's (actually de Moivre's) expansion.
+        //
+        r = 1.0 / x2;
+        value = value + log(x2) - 0.5 * r;
+
+        r = r * r;
+
+        value = value
+            - r * (1.0 / 12.0
+                - r * (1.0 / 120.0
+                    - r * (1.0 / 252.0
+                        - r * (1.0 / 240.0
+                            - r * (1.0 / 132.0)))));
+
+        return value;
+    }
+
 }  // namespace GPBoost
