@@ -25,6 +25,15 @@ namespace GPBoost {
 	/*! \brief Small number that is added in some cases to covariance matrices to make inversion numerically stable */
 	const double EPSILON_ADD_COVARIANCE_STABLE = 1e-10;
 
+	/*! \brief Termination criterion for low-rank pivoted Cholesky decomposition */
+	const double PIV_CHOL_STOP_TOL = 1e-6;
+
+	/*! \brief Threshold for considering a rhs as zero in conjugate gradient algorithms */
+	const double ZERO_RHS_CG_THRESHOLD = 1e-100;
+
+	/*! \brief Threshold for doing reorthogonalization in the Lanczos algorithm */
+	const double LANCZOS_REORTHOGONALIZATION_THRESHOLD = 1e-5;
+
 	/*! \brief Comparing two numbers for equality, source: http://realtimecollisiondetection.net/blog/?p=89 */
 	template <typename T>//T can be double or float
 	inline bool TwoNumbersAreEqual(const T a, const T b) {
